/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 * @author Abner Jose de Faria Silva <abner.silva@indt.org.br>
 * @author Marcio Macedo <marcio.macedo@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string.h>

#include <gtk/gtkbox.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtkentry.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkcombobox.h>
#include <gtk/gtkradiomenuitem.h>
#include <gtk/gtkseparatormenuitem.h>
#include <gtk/gtkmessagedialog.h>
#include <gtk/gtkmain.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkstock.h>
#include <gtk/gtknotebook.h>
#include <glade/glade.h>
#include <glib/gi18n.h>
#include <gconf/gconf-client.h>
#include <gconf/gconf.h>
#include <gdk/gdk.h>

#include <tapioca/base/tapioca_enums.h>

#include "tapiocaui_mainwin.h"
#include "tapiocaui_communication.h"
#include "tapiocaui_buddy_list.h"
#include "tapiocaui_new_buddy_dialog.h"
#include "tapiocaui_new_account_dialog.h"
#include "tapiocaui_utils.h"
#include "tapiocaui_stock_icons.h"
#include "tapiocaui_marshal.h"

#ifdef ENABLE_SYSTRAY
#include "tapiocaui_systray.h"
#endif

#define TAPIOCA_DEFAULT_ACCOUNT "/apps/tapiocaui/default_account"

enum
{
    PROP_0,
    PROP_DB_BUDDY,
    PROP_DB_ACCOUNT
};

enum
{
    LOGIN,
    LOGIN_CANCELLED,
    LOGOUT,
    CALL,
    OPEN_CHAT,
    BUDDY_ADDED,
    BUDDY_REMOVED,
    BUDDY_UPDATED,
    ACCOUNT_STATUS_UPDATED,
    LAST_SIGNAL
};

struct _TapiocaUIMainWinPrivate
{
    TapiocaUIBuddyList *buddy_list;
    TapiocaUIDB *db_account;
    TapiocaUIDB *db_buddy;
    GConfClient *gconf_client;
    GtkNotebook *notebook;
    GtkComboBox *combo_user;
    GtkWidget *event_box_clear;
    GtkWidget *btn_login;
    GtkWidget *btn_cancel_login;
    GtkWidget *lbl_status;
    // GtkWidget *user_menu;
    GtkComboBox *combo_presence;
    GtkEntry *entry_passwd;
    GtkLabel *lbl_login_error_status;
    gchar *user_name;
    guint current_status;
    gchar *current_status_desc;
    GMutex *mutex;
};

typedef struct _PresenceStatus PresenceStatus;

struct _PresenceStatus {
    guint status;
    gchar *status_desc;
    gboolean is_separator;
};

static PresenceStatus presence_status[] = {
    { TAPIOCA_PRESENCE_STATUS_AVAILABLE, NULL, FALSE },
    { TAPIOCA_PRESENCE_STATUS_AWAY, NULL, FALSE },
    { TAPIOCA_PRESENCE_STATUS_DND, NULL, FALSE },
    { TAPIOCA_PRESENCE_STATUS_OFFLINE, NULL, FALSE },
    { -1, NULL, TRUE },
    { -2, NULL, FALSE },
};

static void     tapiocaui_main_win_class_init               (TapiocaUIMainWinClass *k);
static void     tapiocaui_main_win_init                     (TapiocaUIMainWin *self);
static void     tapiocaui_main_win_set_property             (GObject *object,
                                                             guint prop_id,
                                                             const GValue *value,
                                                             GParamSpec *pspec);
static void     tapiocaui_main_win_get_property             (GObject *object,
                                                             guint prop_id,
                                                             GValue *value,
                                                             GParamSpec *pspec);
static void     tapiocaui_main_win_finalize                  (GObject *object);
static void     tapiocaui_main_win_login_cb                 (GtkButton *btn,
                                                             gpointer user_data);
static void     tapiocaui_main_win_cancel_login_cb          (GtkButton *btn,
                                                             gpointer user_data);
static void     tapiocaui_main_win_add_cb                   (GtkButton *btn,
                                                             TapiocaUIMainWin *self);
static void     tapiocaui_main_win_del_cb                   (TapiocaUIBuddyList *list,
                                                             TapiocaUIBuddyData *buddy,
                                                             TapiocaUIMainWin *self);
static void     tapiocaui_main_win_buddy_activated_cb       (TapiocaUIBuddyList *list,
                                                             TapiocaUIBuddyData *buddy,
                                                             gboolean do_call,
                                                             gpointer user_data);
static gboolean tapiocaui_main_win_change_login_cb          (GtkWidget *widget,
                                                             gpointer user_data);
static void     tapiocaui_main_win_account_status_updated_cb
                                                            (GtkComboBox *combo,
                                                             gpointer user_data);
static void     tapiocaui_main_win_account_status_desc_updated_cb
                                                            (GtkEntry *entry,
                                                             gpointer user_data);
static gboolean tapiocaui_main_win_focus_in_status_desc_cb  (GtkWidget *widget,
                                                             GdkEventFocus *event,
                                                             gpointer user_data);
static gboolean tapiocaui_main_win_focus_out_status_desc_cb (GtkWidget *widget,
                                                             GdkEventFocus *event,
                                                             gpointer user_data);
static gboolean is_separator                                (GtkTreeModel *model,
                                                             GtkTreeIter *iter,
                                                             gpointer user_data);

static guint tapiocaui_main_win_signals[LAST_SIGNAL] = { 0 };
static GObjectClass *parent_class = NULL;

G_DEFINE_TYPE (TapiocaUIMainWin, tapiocaui_main_win, GTK_TYPE_WINDOW)

static void
tapiocaui_main_win_class_init (TapiocaUIMainWinClass *k)
{
    GObjectClass *object_class = G_OBJECT_CLASS (k);

    parent_class = g_type_class_peek_parent (k);
    object_class->finalize = tapiocaui_main_win_finalize;
    object_class->set_property = tapiocaui_main_win_set_property;
    object_class->get_property = tapiocaui_main_win_get_property;

    g_object_class_install_property (object_class,
                                     PROP_DB_BUDDY,
                                     g_param_spec_object ("DB-Buddy",
                                                          "TapiocaUIDB",
                                                          "TapiocaUIDB object for buddies",
                                                          G_TYPE_OBJECT,
                                                          G_PARAM_READWRITE));
    g_object_class_install_property (object_class,
                                     PROP_DB_ACCOUNT,
                                     g_param_spec_object ("DB-Account",
                                                          "TapiocaUIDB",
                                                          "TapiocaUIDB object for account",
                                                          G_TYPE_OBJECT,
                                                          G_PARAM_READWRITE));

    tapiocaui_main_win_signals[LOGIN] =
        g_signal_new ("login",
                      TAPIOCAUI_TYPE_MAIN_WIN,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIMainWinClass, login),
                      NULL, NULL,
                      tapiocaui_marshal_VOID__STRING_STRING,
                      G_TYPE_NONE,
                      2,
                      G_TYPE_STRING,
                      G_TYPE_STRING);
    tapiocaui_main_win_signals[LOGIN_CANCELLED] =
        g_signal_new ("login-cancelled",
                      TAPIOCAUI_TYPE_MAIN_WIN,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIMainWinClass, login_cancelled),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE,
                      0);
    tapiocaui_main_win_signals[LOGOUT] =
        g_signal_new ("logout",
                      TAPIOCAUI_TYPE_MAIN_WIN,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIMainWinClass, logout),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE,
                      0);
    tapiocaui_main_win_signals[CALL] =
        g_signal_new ("call",
                      TAPIOCAUI_TYPE_MAIN_WIN,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIMainWinClass, call),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);
    tapiocaui_main_win_signals[OPEN_CHAT] =
        g_signal_new ("open-chat",
                      TAPIOCAUI_TYPE_MAIN_WIN,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIMainWinClass, open_chat),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);
    tapiocaui_main_win_signals[BUDDY_ADDED] =
        g_signal_new ("buddy-added",
                      TAPIOCAUI_TYPE_MAIN_WIN,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIMainWinClass, buddy_added),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);
    tapiocaui_main_win_signals[BUDDY_REMOVED] =
        g_signal_new ("buddy-removed",
                      TAPIOCAUI_TYPE_MAIN_WIN,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIMainWinClass, buddy_removed),
                      NULL, NULL,
                      g_cclosure_marshal_VOID__STRING,
                      G_TYPE_NONE,
                      1,
                      G_TYPE_STRING);
    tapiocaui_main_win_signals[BUDDY_UPDATED] =
        g_signal_new ("buddy-updated",
                      TAPIOCAUI_TYPE_MAIN_WIN,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIMainWinClass, buddy_updated),
                      NULL, NULL,
                      tapiocaui_marshal_VOID__STRING_STRING,
                      G_TYPE_NONE,
                      2,
                      G_TYPE_STRING,
                      G_TYPE_STRING);
    tapiocaui_main_win_signals[ACCOUNT_STATUS_UPDATED] =
        g_signal_new ("account-status-updated",
                      TAPIOCAUI_TYPE_MAIN_WIN,
                      G_SIGNAL_RUN_LAST,
                      G_STRUCT_OFFSET (TapiocaUIMainWinClass, account_status_updated),
                      NULL, NULL,
                      tapiocaui_marshal_VOID__UINT_STRING,
                      G_TYPE_NONE,
                      2,
                      G_TYPE_UINT,
                      G_TYPE_STRING);
}

static void
tapiocaui_main_win_init (TapiocaUIMainWin *self)
{
    GladeXML *xml;
    GtkWidget *tmp;
    GdkColor color;
    GtkListStore *supported_presences;
    GtkTreeIter iter;
    guint i, presence_count;

    gtk_window_set_title (GTK_WINDOW (self), "Tapioca");
    gtk_window_set_default_icon_from_file (tapiocaui_file ("tapioca_icon_small.png"),
                                           NULL);

    self->priv = g_new0 (TapiocaUIMainWinPrivate, 1);
    self->priv->mutex = g_mutex_new ();

    tapiocaui_stock_icons_init ();
    self->priv->gconf_client = gconf_client_get_default ();

    xml = glade_xml_new (tapiocaui_file ("main-window.glade"), "vbox_main", NULL);
    glade_xml_signal_autoconnect (xml);
    tmp = glade_xml_get_widget (xml, "vbox_main");
    gtk_container_add (GTK_CONTAINER (self), tmp);

    self->priv->notebook = GTK_NOTEBOOK (glade_xml_get_widget (xml, "notebook"));
    gtk_notebook_set_show_tabs (self->priv->notebook, FALSE);

    // Login screen
    self->priv->combo_user = GTK_COMBO_BOX (glade_xml_get_widget (xml, "combo_user"));
    g_signal_connect (G_OBJECT (self->priv->combo_user),
                      "changed",
                      G_CALLBACK (tapiocaui_main_win_change_login_cb),
                      self);

    self->priv->entry_passwd = GTK_ENTRY (glade_xml_get_widget (xml, "entry_passwd"));
    g_signal_connect (G_OBJECT (self->priv->entry_passwd),
                      "activate",
                      G_CALLBACK (tapiocaui_main_win_login_cb),
                      self);

    self->priv->btn_login = glade_xml_get_widget (xml, "btn_login");
    g_signal_connect (G_OBJECT (self->priv->btn_login),
                      "clicked",
                      G_CALLBACK (tapiocaui_main_win_login_cb),
                      self);

    self->priv->lbl_login_error_status = GTK_LABEL (glade_xml_get_widget (xml, "lbl_login_error_status"));
    color.red = 65535;
    color.green = color.blue = 0;
    gtk_label_set_justify(GTK_LABEL(self->priv->lbl_login_error_status), GTK_JUSTIFY_LEFT);
    gtk_widget_modify_fg (GTK_WIDGET (self->priv->lbl_login_error_status),
                          GTK_STATE_NORMAL,
                          &color);

    // login status
    self->priv->btn_cancel_login = glade_xml_get_widget (xml, "btn_cancel_login");
    g_signal_connect (G_OBJECT (self->priv->btn_cancel_login),
                      "clicked",
                      G_CALLBACK (tapiocaui_main_win_cancel_login_cb),
                      self);

    // buddy list
    self->priv->lbl_status = glade_xml_get_widget (xml, "lbl_status");

    tmp = glade_xml_get_widget (xml, "img_top");
    gtk_image_set_from_file (GTK_IMAGE (tmp), tapiocaui_file ("tapioca_title.png"));

    // Create user menu
    self->priv->combo_presence = GTK_COMBO_BOX (glade_xml_get_widget (xml, "combo_presence"));
    gtk_combo_box_set_row_separator_func (self->priv->combo_presence,
            is_separator, NULL, NULL);

    tmp = GTK_WIDGET (gtk_bin_get_child (GTK_BIN (self->priv->combo_presence)));
    g_signal_connect (G_OBJECT (self->priv->combo_presence),
                      "changed",
                      G_CALLBACK (tapiocaui_main_win_account_status_updated_cb),
                      self);
    g_signal_connect (G_OBJECT (tmp),
                      "activate",
                      G_CALLBACK (tapiocaui_main_win_account_status_desc_updated_cb),
                      self);
    g_signal_connect (G_OBJECT (tmp),
                      "focus-in-event",
                      G_CALLBACK (tapiocaui_main_win_focus_in_status_desc_cb),
                      self);
    g_signal_connect (G_OBJECT (tmp),
                      "focus-out-event",
                      G_CALLBACK (tapiocaui_main_win_focus_out_status_desc_cb),
                      self);

    presence_status[0].status_desc = _("Available");
    presence_status[1].status_desc = _("Away From Computer");
    presence_status[2].status_desc = _("Busy");
    presence_status[3].status_desc = _("Hidden");
    presence_status[4].status_desc = "";
    presence_status[5].status_desc = _("Log Out");

    supported_presences = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_BOOLEAN);

    presence_count = sizeof (presence_status) / sizeof (PresenceStatus);
    for (i = 0; i < presence_count; ++i) {
        gtk_list_store_append (supported_presences, &iter);
        gtk_list_store_set (supported_presences, &iter,
                0, presence_status[i].status_desc,
                1, presence_status[i].is_separator, -1);
    }

    gtk_combo_box_set_model (self->priv->combo_presence, GTK_TREE_MODEL (supported_presences));

    tmp = glade_xml_get_widget (xml, "box_buddy");
    self->priv->buddy_list = tapiocaui_buddy_list_new ();
    g_signal_connect (G_OBJECT (self->priv->buddy_list),
                      "buddy-activated",
                      G_CALLBACK (tapiocaui_main_win_buddy_activated_cb),
                      self);
    g_signal_connect (G_OBJECT (self->priv->buddy_list),
                      "buddy-remove-requested",
                      G_CALLBACK (tapiocaui_main_win_del_cb),
                      self);

    gtk_box_pack_start_defaults (GTK_BOX (tmp), GTK_WIDGET (self->priv->buddy_list));
    gtk_box_reorder_child (GTK_BOX (tmp),
                           GTK_WIDGET (self->priv->buddy_list), 1);

    tmp = glade_xml_get_widget (xml, "img_add");
    gtk_image_set_from_file (GTK_IMAGE (tmp), tapiocaui_file ("icon_add_contact.png"));
    tmp = glade_xml_get_widget (xml, "btn_add");
    g_signal_connect (G_OBJECT (tmp),
                      "clicked",
                      G_CALLBACK (tapiocaui_main_win_add_cb),
                      self);

    gtk_widget_show_all (GTK_WIDGET (self));

#ifdef ENABLE_SYSTRAY
    tapiocaui_systray_init(self);
#endif

    g_object_unref (G_OBJECT (xml));
}

static void
tapiocaui_main_win_set_property (GObject *object,
                                 guint prop_id,
                                 const GValue *value,
                                 GParamSpec *pspec)
{
    TapiocaUIMainWin *self = TAPIOCAUI_MAIN_WIN (object);
    g_return_if_fail (value != NULL);
    switch (prop_id)
    {
        case PROP_DB_ACCOUNT:
            self->priv->db_account = g_value_get_object (value);
            break;
        case PROP_DB_BUDDY:
            if (self->priv->db_buddy) {
                tapiocaui_buddy_list_disconnect_db (self->priv->buddy_list);
            }
            self->priv->db_buddy = g_value_get_object (value);
            tapiocaui_buddy_list_connect_db (self->priv->buddy_list, self->priv->db_buddy);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void
tapiocaui_main_win_get_property (GObject *object,
                                 guint prop_id,
                                 GValue *value,
                                 GParamSpec *pspec)
{
    TapiocaUIMainWin *self = TAPIOCAUI_MAIN_WIN (object);
    switch (prop_id)
    {
        case PROP_DB_ACCOUNT:
            g_value_set_object (value, self->priv->db_account);
            break;
        case PROP_DB_BUDDY:
            g_value_set_object (value, self->priv->db_buddy);
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
            break;
    }
}

static void
tapiocaui_main_win_finalize (GObject *object)
{
    TapiocaUIMainWin *self;
    g_return_if_fail (object != NULL);
    g_return_if_fail (TAPIOCAUI_IS_MAIN_WIN (object));

    self = TAPIOCAUI_MAIN_WIN (object);
    self->priv->db_account = NULL;
    self->priv->db_buddy = NULL;
    g_object_unref (G_OBJECT (self->priv->gconf_client));

    g_mutex_free (self->priv->mutex);
    g_free (self->priv);

    G_OBJECT_CLASS (parent_class)->finalize (object);
}

TapiocaUIMainWin *
tapiocaui_main_win_new (TapiocaUIDB *db)
{
    TapiocaUIMainWin *self;
    self = g_object_new (TAPIOCAUI_TYPE_MAIN_WIN, "DB-Account", db, NULL);
    tapiocaui_main_win_login (self);
    return self;
}

static void
tapiocaui_main_win_add_cb (GtkButton *btn,
                           TapiocaUIMainWin *self)
{
    gchar *buddy_id = NULL;
    GtkDialog *new_buddy = GTK_DIALOG (tapiocaui_new_buddy_dialog_new (self->priv->db_buddy,
                                                                       FALSE));

    gtk_dialog_run (new_buddy);

    buddy_id = (gchar *) tapiocaui_new_buddy_dialog_get_id (TAPIOCAUI_NEW_BUDDY_DIALOG (new_buddy));

    if (buddy_id)
        g_signal_emit (G_OBJECT (self), tapiocaui_main_win_signals [BUDDY_ADDED], 0, buddy_id);

    gtk_widget_destroy (GTK_WIDGET (new_buddy));
}

static void
tapiocaui_main_win_del_cb (TapiocaUIBuddyList *list,
                           TapiocaUIBuddyData *buddy,
                           TapiocaUIMainWin *self)
{
    GtkWidget *msg;

    msg = gtk_message_dialog_new_with_markup (GTK_WINDOW (self),
                                              GTK_DIALOG_MODAL,
                                              GTK_MESSAGE_QUESTION,
                                              GTK_BUTTONS_NONE,
                                              _("Are you sure you want to delete buddy <b>%s</b>?"),
                                              buddy->name);

    gtk_dialog_add_buttons (GTK_DIALOG (msg),
                            GTK_STOCK_NO, GTK_RESPONSE_CANCEL,
                            GTK_STOCK_YES, GTK_RESPONSE_OK, NULL);
    gtk_dialog_set_default_response (GTK_DIALOG (msg), GTK_RESPONSE_NO);

    if (gtk_dialog_run (GTK_DIALOG (msg)) == GTK_RESPONSE_OK) {
        tapiocaui_db_remove_buddy (self->priv->db_buddy, buddy);
        g_signal_emit (G_OBJECT (self), tapiocaui_main_win_signals [BUDDY_REMOVED], 0, buddy->id);
        tapiocaui_db_entry_free (buddy);
    }

    gtk_widget_destroy (GTK_WIDGET (msg));
    tapiocaui_db_commit (self->priv->db_buddy);

}

static void
tapiocaui_main_win_buddy_activated_cb (TapiocaUIBuddyList *list,
                                       TapiocaUIBuddyData *buddy,
                                       gboolean do_call,
                                       gpointer user_data)
{
    TapiocaUIMainWin *self = TAPIOCAUI_MAIN_WIN (user_data);

    if (do_call)
        g_signal_emit (G_OBJECT (self), tapiocaui_main_win_signals [CALL], 0, buddy->id);
    else
        g_signal_emit (G_OBJECT (self), tapiocaui_main_win_signals [OPEN_CHAT], 0, buddy->id);
}

void
tapiocaui_main_win_login (TapiocaUIMainWin *self)
{
    GList *lst;
    GtkListStore *accounts;
    GtkTreeIter iter;
    GError *error = NULL;
    gboolean have_accounts;

    have_accounts = FALSE;
    gtk_notebook_set_current_page (self->priv->notebook, 0);
    gtk_widget_grab_focus (GTK_WIDGET (self->priv->combo_user));
    gtk_widget_grab_default (self->priv->btn_login);
    gtk_entry_set_text (self->priv->entry_passwd, "");

    accounts = gtk_list_store_new (1, G_TYPE_STRING);

    lst = tapiocaui_db_get_buddy_list (self->priv->db_account, TAPIOCAUI_DB_ENTRY_ACCOUNT);
    if (lst)
        have_accounts = TRUE;
    while (lst) {
        TapiocaUIBuddyData *entry = (TapiocaUIBuddyData *) lst->data;
        gtk_list_store_append (accounts, &iter);
        gtk_list_store_set (accounts, &iter, 0, entry->id, -1);
        lst = lst->next;
    }
    gtk_combo_box_set_model (self->priv->combo_user, GTK_TREE_MODEL (accounts));

    if (!have_accounts) {
        gconf_client_set_int (self->priv->gconf_client,
                                 TAPIOCA_DEFAULT_ACCOUNT,
                                 0, &error);
        return;
    }
    else {
        gtk_widget_grab_focus (GTK_WIDGET (self->priv->entry_passwd));
    }

    gtk_combo_box_set_active (self->priv->combo_user, 0);
}

void
tapiocaui_main_win_cancel_login (TapiocaUIMainWin *self,
                                 gint status)
{
    gtk_notebook_set_current_page (self->priv->notebook, 0);
    switch (status) {
        // TODO treat error
        default:
            break;
    }
}

void
tapiocaui_main_win_load_buddy (TapiocaUIMainWin *self,
                               TapiocaUIDB *db,
                               const gchar *user)
{
    GdkColor color;
    gchar *status;
    TapiocaUIBuddyData *entry;

    gtk_notebook_set_current_page (self->priv->notebook, 2);
    gtk_combo_box_set_active (self->priv->combo_presence, 0);

    g_object_set (G_OBJECT (self), "DB-Buddy", db, NULL);

    entry = tapiocaui_db_find_buddy_by_id (self->priv->db_account,
                                           TAPIOCAUI_DB_ENTRY_ACCOUNT,
                                           user);
    g_return_if_fail (entry != NULL);

    status = g_strdup_printf ("<b>%s</b>", entry->name);
    gtk_label_set_markup (GTK_LABEL (self->priv->lbl_status), status);
    g_free (status);
    color.green = color.blue = color.red = 32000;
    gtk_widget_grab_focus (GTK_WIDGET (self->priv->buddy_list));
}

void
tapiocaui_main_win_logout (TapiocaUIMainWin *self)
{
    if (self->priv->user_name != NULL) {
        g_free (self->priv->user_name);
        self->priv->user_name = NULL;
    }
    tapiocaui_main_win_login (self);
}

static void
tapiocaui_main_win_login_cb (GtkButton *btn,
                             gpointer user_data)
{
    TapiocaUIMainWin *self = TAPIOCAUI_MAIN_WIN (user_data);
    gchar *user;
    TapiocaUIBuddyData *entry;
    gint index;

    // change to page login and start login animation
    gtk_notebook_set_current_page (self->priv->notebook, 1);

    user = gtk_combo_box_get_active_text (self->priv->combo_user);
    entry = tapiocaui_db_find_buddy_by_id (self->priv->db_account, TAPIOCAUI_DB_ENTRY_ACCOUNT, user);
    if (entry == NULL) {
        TapiocaUIBuddyData *account;
        account = tapiocaui_db_entry_new (TAPIOCAUI_DB_ENTRY_ACCOUNT);
        account->id = g_strstrip (g_strdup (user));
        account->name = g_strdup (account->id);
        tapiocaui_db_insert_buddy (self->priv->db_account, account);
        tapiocaui_db_commit (self->priv->db_account);
        entry = account;
    }

    index = gtk_combo_box_get_active (self->priv->combo_user);
    if (self->priv->user_name != NULL)
        g_free (self->priv->user_name);
    self->priv->user_name = g_strdup (entry->id);

    g_signal_emit (G_OBJECT (self), tapiocaui_main_win_signals [LOGIN], 0,
                   user,
                   gtk_entry_get_text (self->priv->entry_passwd));
    g_free (user);
}

static void
tapiocaui_main_win_cancel_login_cb (GtkButton *btn,
                                    gpointer user_data)
{
    TapiocaUIMainWin *self = TAPIOCAUI_MAIN_WIN (user_data);

    g_signal_emit (G_OBJECT (self), tapiocaui_main_win_signals [LOGIN_CANCELLED], 0);
    gtk_notebook_set_current_page (self->priv->notebook, 0);
}

TapiocaUICommunication*
tapiocaui_main_win_open_chat (TapiocaUIMainWin *self,
                              TapiocaUIBuddyData *buddy)
{
    GtkWidget *comm = NULL;
    comm = tapiocaui_communication_new (tapiocaui_main_win_get_user (self), buddy);

#ifdef ENABLE_SYSTRAY
    if((gdk_window_get_state(GTK_WIDGET(self)->window) &
                        GDK_WINDOW_STATE_ICONIFIED) ||
                        !GTK_WIDGET_VISIBLE(self))
    {
        tapiocaui_systray_add_hidden_window(GTK_WIDGET (comm));
    }
    else
    {
#endif
        gtk_widget_show (GTK_WIDGET (comm));
        gtk_window_set_urgency_hint (GTK_WINDOW (comm), TRUE);
#ifdef ENABLE_SYSTRAY        
    }
#endif

    return TAPIOCAUI_COMMUNICATION (comm);
}

static gboolean
tapiocaui_main_win_change_login_cb (GtkWidget *widget,
                                    gpointer user_data)
{
    TapiocaUIMainWin *self = TAPIOCAUI_MAIN_WIN (user_data);
    gchar *user = gtk_combo_box_get_active_text (self->priv->combo_user);

    g_strstrip (user);

    if (g_utf8_strlen (user, -1) > 0) {
        TapiocaUIBuddyData *entry;

        entry = tapiocaui_db_find_buddy_by_id (self->priv->db_account, TAPIOCAUI_DB_ENTRY_ACCOUNT,
                                                user);
    }

    g_free (user);

    return FALSE;
}

const gchar *
tapiocaui_main_win_get_user (TapiocaUIMainWin   *self)
{
    return self->priv->user_name;
}

void
tapiocaui_main_win_update_buddy (TapiocaUIMainWin *self,
                                 TapiocaUIBuddyData *buddy)
{
    g_return_if_fail (self != NULL);

    tapiocaui_buddy_list_update_buddy (self->priv->buddy_list, buddy);
}

void tapiocaui_main_win_set_error_message (TapiocaUIMainWin *self, gchar *message)
{
    g_return_if_fail (self != NULL);
    gtk_label_set_markup(GTK_LABEL(self->priv->lbl_login_error_status) , message);
}

static void
tapiocaui_main_win_account_status_updated_cb (GtkComboBox *combo,
                                              gpointer user_data)
{
    TapiocaUIMainWin *self = TAPIOCAUI_MAIN_WIN (user_data);
    guint status;
    const gchar *status_desc = "";

    status = gtk_combo_box_get_active (combo);

    if (status == -1)
        return;

    g_mutex_lock (self->priv->mutex);

    if (self->priv->current_status == status &&
        !self->priv->current_status_desc) {
        g_mutex_unlock (self->priv->mutex);
        return;
    }

    if (status == 5) {
        tapiocaui_db_commit (self->priv->db_buddy);
        g_mutex_unlock (self->priv->mutex);
        g_signal_emit (G_OBJECT (self), tapiocaui_main_win_signals [LOGOUT], 0);
        return;
    }

    self->priv->current_status = status;
    if (self->priv->current_status_desc) {
        g_free (self->priv->current_status_desc);
        self->priv->current_status_desc = NULL;
    }

    g_signal_emit (G_OBJECT (self), tapiocaui_main_win_signals[ACCOUNT_STATUS_UPDATED],
            0, presence_status[status].status, status_desc);

    g_mutex_unlock (self->priv->mutex);
}

static void
tapiocaui_main_win_account_status_desc_updated_cb (GtkEntry *entry,
                                                   gpointer user_data)
{
    TapiocaUIMainWin *self = TAPIOCAUI_MAIN_WIN (user_data);
    gchar *status_desc;

    g_mutex_lock (self->priv->mutex);

    status_desc = g_strdup (gtk_entry_get_text (entry));
    if (g_strcasecmp (presence_status[self->priv->current_status].status_desc,
                      status_desc) != 0) {
        g_signal_emit (G_OBJECT (self),
                tapiocaui_main_win_signals[ACCOUNT_STATUS_UPDATED],
                0, presence_status[self->priv->current_status].status,
                status_desc);
        if (self->priv->current_status_desc)
            g_free (self->priv->current_status_desc);
        self->priv->current_status_desc = status_desc;
    }
    else {
        if (self->priv->current_status_desc) {
            g_free (status_desc);
            status_desc = "";

            g_free (self->priv->current_status_desc);
            self->priv->current_status_desc = NULL;
            g_signal_emit (G_OBJECT (self),
                    tapiocaui_main_win_signals[ACCOUNT_STATUS_UPDATED],
                    0, presence_status[self->priv->current_status].status,
                    status_desc);
        }
        else {
            g_free (status_desc);
        }
    }

    g_mutex_unlock (self->priv->mutex);
}

static gboolean
tapiocaui_main_win_focus_in_status_desc_cb (GtkWidget *widget,
                                            GdkEventFocus *event,
                                            gpointer user_data)
{
    gtk_entry_set_text (GTK_ENTRY (widget), "");
    return FALSE;
}

static gboolean
tapiocaui_main_win_focus_out_status_desc_cb (GtkWidget *widget,
                                             GdkEventFocus *event,
                                             gpointer user_data)
{
    TapiocaUIMainWin *self = (TapiocaUIMainWin *) user_data;

    g_mutex_lock (self->priv->mutex);
    if (g_utf8_strlen (gtk_entry_get_text (GTK_ENTRY (widget)), -1) == 0) {
        if (self->priv->current_status_desc) {
            gtk_entry_set_text (GTK_ENTRY (widget), self->priv->current_status_desc);
            g_mutex_unlock (self->priv->mutex);
        }
        else {
            g_mutex_unlock (self->priv->mutex);
            gtk_combo_box_set_active (self->priv->combo_presence,
                    self->priv->current_status);
        }
    }
    else {
        if (!self->priv->current_status_desc) {
            g_mutex_unlock (self->priv->mutex);
            tapiocaui_main_win_account_status_desc_updated_cb (
                    GTK_ENTRY (widget),
                    self);
        }
        else {
            g_mutex_unlock (self->priv->mutex);
        }
    }

    return FALSE;
}

static gboolean
is_separator (GtkTreeModel *model,
              GtkTreeIter *iter,
              gpointer user_data)
{
    gboolean result;
    gtk_tree_model_get (model, iter, 1, &result, -1);
    return result;
}
